// Copyright 2016 PDFium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Original code copyright 2014 Foxit Software Inc. http://www.foxitsoftware.com

#include "xfa/fxfa/parser/xfa_basic_data.h"

#include "fxjs/xfa/cjx_arc.h"
#include "fxjs/xfa/cjx_area.h"
#include "fxjs/xfa/cjx_assist.h"
#include "fxjs/xfa/cjx_barcode.h"
#include "fxjs/xfa/cjx_bind.h"
#include "fxjs/xfa/cjx_binditems.h"
#include "fxjs/xfa/cjx_bookend.h"
#include "fxjs/xfa/cjx_boolean.h"
#include "fxjs/xfa/cjx_border.h"
#include "fxjs/xfa/cjx_break.h"
#include "fxjs/xfa/cjx_breakafter.h"
#include "fxjs/xfa/cjx_breakbefore.h"
#include "fxjs/xfa/cjx_button.h"
#include "fxjs/xfa/cjx_calculate.h"
#include "fxjs/xfa/cjx_caption.h"
#include "fxjs/xfa/cjx_certificate.h"
#include "fxjs/xfa/cjx_certificates.h"
#include "fxjs/xfa/cjx_checkbutton.h"
#include "fxjs/xfa/cjx_choicelist.h"
#include "fxjs/xfa/cjx_color.h"
#include "fxjs/xfa/cjx_comb.h"
#include "fxjs/xfa/cjx_command.h"
#include "fxjs/xfa/cjx_connect.h"
#include "fxjs/xfa/cjx_connectstring.h"
#include "fxjs/xfa/cjx_contentarea.h"
#include "fxjs/xfa/cjx_corner.h"
#include "fxjs/xfa/cjx_datavalue.h"
#include "fxjs/xfa/cjx_datawindow.h"
#include "fxjs/xfa/cjx_date.h"
#include "fxjs/xfa/cjx_datetime.h"
#include "fxjs/xfa/cjx_datetimeedit.h"
#include "fxjs/xfa/cjx_decimal.h"
#include "fxjs/xfa/cjx_defaultui.h"
#include "fxjs/xfa/cjx_delete.h"
#include "fxjs/xfa/cjx_delta.h"
#include "fxjs/xfa/cjx_desc.h"
#include "fxjs/xfa/cjx_digestmethod.h"
#include "fxjs/xfa/cjx_digestmethods.h"
#include "fxjs/xfa/cjx_draw.h"
#include "fxjs/xfa/cjx_edge.h"
#include "fxjs/xfa/cjx_encoding.h"
#include "fxjs/xfa/cjx_encodings.h"
#include "fxjs/xfa/cjx_encrypt.h"
#include "fxjs/xfa/cjx_event.h"
#include "fxjs/xfa/cjx_eventpseudomodel.h"
#include "fxjs/xfa/cjx_exclgroup.h"
#include "fxjs/xfa/cjx_exdata.h"
#include "fxjs/xfa/cjx_execute.h"
#include "fxjs/xfa/cjx_exobject.h"
#include "fxjs/xfa/cjx_extras.h"
#include "fxjs/xfa/cjx_field.h"
#include "fxjs/xfa/cjx_fill.h"
#include "fxjs/xfa/cjx_filter.h"
#include "fxjs/xfa/cjx_float.h"
#include "fxjs/xfa/cjx_font.h"
#include "fxjs/xfa/cjx_format.h"
#include "fxjs/xfa/cjx_handler.h"
#include "fxjs/xfa/cjx_hostpseudomodel.h"
#include "fxjs/xfa/cjx_image.h"
#include "fxjs/xfa/cjx_imageedit.h"
#include "fxjs/xfa/cjx_insert.h"
#include "fxjs/xfa/cjx_instancemanager.h"
#include "fxjs/xfa/cjx_integer.h"
#include "fxjs/xfa/cjx_issuers.h"
#include "fxjs/xfa/cjx_items.h"
#include "fxjs/xfa/cjx_keep.h"
#include "fxjs/xfa/cjx_keyusage.h"
#include "fxjs/xfa/cjx_layoutpseudomodel.h"
#include "fxjs/xfa/cjx_line.h"
#include "fxjs/xfa/cjx_linear.h"
#include "fxjs/xfa/cjx_logpseudomodel.h"
#include "fxjs/xfa/cjx_manifest.h"
#include "fxjs/xfa/cjx_map.h"
#include "fxjs/xfa/cjx_margin.h"
#include "fxjs/xfa/cjx_mdp.h"
#include "fxjs/xfa/cjx_medium.h"
#include "fxjs/xfa/cjx_message.h"
#include "fxjs/xfa/cjx_node.h"
#include "fxjs/xfa/cjx_numericedit.h"
#include "fxjs/xfa/cjx_occur.h"
#include "fxjs/xfa/cjx_oid.h"
#include "fxjs/xfa/cjx_oids.h"
#include "fxjs/xfa/cjx_operation.h"
#include "fxjs/xfa/cjx_overflow.h"
#include "fxjs/xfa/cjx_packet.h"
#include "fxjs/xfa/cjx_pagearea.h"
#include "fxjs/xfa/cjx_pageset.h"
#include "fxjs/xfa/cjx_para.h"
#include "fxjs/xfa/cjx_password.h"
#include "fxjs/xfa/cjx_passwordedit.h"
#include "fxjs/xfa/cjx_pattern.h"
#include "fxjs/xfa/cjx_picture.h"
#include "fxjs/xfa/cjx_query.h"
#include "fxjs/xfa/cjx_radial.h"
#include "fxjs/xfa/cjx_reason.h"
#include "fxjs/xfa/cjx_reasons.h"
#include "fxjs/xfa/cjx_recordset.h"
#include "fxjs/xfa/cjx_rectangle.h"
#include "fxjs/xfa/cjx_ref.h"
#include "fxjs/xfa/cjx_rootelement.h"
#include "fxjs/xfa/cjx_script.h"
#include "fxjs/xfa/cjx_select.h"
#include "fxjs/xfa/cjx_setproperty.h"
#include "fxjs/xfa/cjx_signature.h"
#include "fxjs/xfa/cjx_signatureproperties.h"
#include "fxjs/xfa/cjx_signaturepseudomodel.h"
#include "fxjs/xfa/cjx_signdata.h"
#include "fxjs/xfa/cjx_signing.h"
#include "fxjs/xfa/cjx_soapaction.h"
#include "fxjs/xfa/cjx_soapaddress.h"
#include "fxjs/xfa/cjx_solid.h"
#include "fxjs/xfa/cjx_source.h"
#include "fxjs/xfa/cjx_sourceset.h"
#include "fxjs/xfa/cjx_speak.h"
#include "fxjs/xfa/cjx_stipple.h"
#include "fxjs/xfa/cjx_subform.h"
#include "fxjs/xfa/cjx_subformset.h"
#include "fxjs/xfa/cjx_subjectdn.h"
#include "fxjs/xfa/cjx_subjectdns.h"
#include "fxjs/xfa/cjx_submit.h"
#include "fxjs/xfa/cjx_text.h"
#include "fxjs/xfa/cjx_textedit.h"
#include "fxjs/xfa/cjx_time.h"
#include "fxjs/xfa/cjx_timestamp.h"
#include "fxjs/xfa/cjx_tooltip.h"
#include "fxjs/xfa/cjx_traversal.h"
#include "fxjs/xfa/cjx_traverse.h"
#include "fxjs/xfa/cjx_tree.h"
#include "fxjs/xfa/cjx_treelist.h"
#include "fxjs/xfa/cjx_ui.h"
#include "fxjs/xfa/cjx_update.h"
#include "fxjs/xfa/cjx_uri.h"
#include "fxjs/xfa/cjx_user.h"
#include "fxjs/xfa/cjx_validate.h"
#include "fxjs/xfa/cjx_value.h"
#include "fxjs/xfa/cjx_variables.h"
#include "fxjs/xfa/cjx_wsdladdress.h"
#include "fxjs/xfa/cjx_wsdlconnection.h"
#include "fxjs/xfa/cjx_xfa.h"
#include "fxjs/xfa/cjx_xmlconnection.h"
#include "fxjs/xfa/cjx_xsdconnection.h"
#include "xfa/fxfa/fxfa_basic.h"

namespace {

const XFA_AttributeValueInfo g_XFAEnumData[] = {
#undef VALUE____
#define VALUE____(a, b, c) {a, XFA_AttributeValue::c, b},
#include "xfa/fxfa/parser/attribute_values.inc"
#undef VALUE____
};

const XFA_Element g_XFAScriptParents[] = {
#undef ELEM____
#undef ELEM_HIDDEN____
#define ELEM____(a, b, c, d) XFA_Element::d,
#define ELEM_HIDDEN____(a, b) XFA_Element::b,
#include "xfa/fxfa/parser/elements.inc"
#undef ELEM____
#undef ELEM_HIDDEN____
};

struct ScriptIndexRecord {
  uint16_t wAttributeStart;
  uint16_t wAttributeCount;
};

const ScriptIndexRecord g_ScriptIndexTable[] = {
    {/* ps */ 0, 2},
    {/* to */ 2, 2},
    {/* ui */ 4, 2},
    {/* recordSet */ 6, 8},
    {/* subsetBelow */ 14, 4},
    {/* subformSet */ 18, 5},
    {/* adobeExtensionLevel */ 23, 2},
    {/* typeface */ 25, 1},
    {/* break */ 26, 12},
    {/* fontInfo */ 38, 2},
    {/* numberPattern */ 40, 1},
    {/* dynamicRender */ 41, 3},
    {/* printScaling */ 44, 2},
    {/* checkButton */ 46, 6},
    {/* datePatterns */ 52, 0},
    {/* sourceSet */ 52, 2},
    {/* amd */ 54, 2},
    {/* arc */ 56, 6},
    {/* day */ 62, 0},
    {/* era */ 62, 0},
    {/* jog */ 62, 2},
    {/* log */ 64, 2},
    {/* map */ 66, 6},
    {/* mdp */ 72, 4},
    {/* breakBefore */ 76, 7},
    {/* oid */ 83, 2},
    {/* pcl */ 85, 3},
    {/* pdf */ 88, 3},
    {/* ref */ 91, 2},
    {/* uri */ 93, 6},
    {/* xdc */ 99, 4},
    {/* xdp */ 103, 2},
    {/* xfa */ 105, 3},
    {/* xsl */ 108, 4},
    {/* zpl */ 112, 3},
    {/* cache */ 115, 2},
    {/* margin */ 117, 6},
    {/* keyUsage */ 123, 12},
    {/* exclude */ 135, 2},
    {/* choiceList */ 137, 5},
    {/* level */ 142, 2},
    {/* labelPrinter */ 144, 3},
    {/* calendarSymbols */ 147, 1},
    {/* para */ 148, 14},
    {/* part */ 162, 2},
    {/* pdfa */ 164, 2},
    {/* filter */ 166, 3},
    {/* present */ 169, 2},
    {/* pagination */ 171, 2},
    {/* encoding */ 173, 2},
    {/* event */ 175, 4},
    {/* whitespace */ 179, 2},
    {/* defaultUi */ 181, 2},
    {/* dataModel */ 183, 0},
    {/* barcode */ 183, 20},
    {/* timePattern */ 203, 1},
    {/* batchOutput */ 204, 3},
    {/* enforce */ 207, 2},
    {/* currencySymbols */ 209, 0},
    {/* addSilentPrint */ 209, 2},
    {/* rename */ 211, 2},
    {/* operation */ 213, 4},
    {/* typefaces */ 217, 0},
    {/* subjectDNs */ 217, 1},
    {/* issuers */ 218, 3},
    {/* signaturePseudoModel */ 221, 0},
    {/* wsdlConnection */ 221, 1},
    {/* debug */ 222, 2},
    {/* delta */ 224, 3},
    {/* eraNames */ 227, 0},
    {/* modifyAnnots */ 227, 2},
    {/* startNode */ 229, 2},
    {/* button */ 231, 3},
    {/* format */ 234, 2},
    {/* border */ 236, 6},
    {/* area */ 242, 10},
    {/* hyphenation */ 252, 9},
    {/* text */ 261, 5},
    {/* time */ 266, 4},
    {/* type */ 270, 2},
    {/* overprint */ 272, 2},
    {/* certificates */ 274, 5},
    {/* encryptionMethods */ 279, 3},
    {/* setProperty */ 282, 2},
    {/* printerName */ 284, 2},
    {/* startPage */ 286, 2},
    {/* pageOffset */ 288, 2},
    {/* dateTime */ 290, 4},
    {/* comb */ 294, 3},
    {/* pattern */ 297, 3},
    {/* ifEmpty */ 300, 2},
    {/* suppressBanner */ 302, 2},
    {/* outputBin */ 304, 2},
    {/* field */ 306, 36},
    {/* agent */ 342, 3},
    {/* outputXSL */ 345, 2},
    {/* adjustData */ 347, 2},
    {/* autoSave */ 349, 2},
    {/* contentArea */ 351, 7},
    {/* eventPseudoModel */ 358, 17},
    {/* wsdlAddress */ 375, 2},
    {/* solid */ 377, 2},
    {/* dateTimeSymbols */ 379, 0},
    {/* encryptionLevel */ 379, 2},
    {/* edge */ 381, 6},
    {/* stipple */ 387, 3},
    {/* attributes */ 390, 2},
    {/* versionControl */ 392, 4},
    {/* meridiem */ 396, 0},
    {/* exclGroup */ 396, 30},
    {/* toolTip */ 426, 2},
    {/* compress */ 428, 3},
    {/* reason */ 431, 2},
    {/* execute */ 433, 5},
    {/* contentCopy */ 438, 2},
    {/* dateTimeEdit */ 440, 3},
    {/* config */ 443, 2},
    {/* image */ 445, 8},
    {/* #xHTML */ 453, 1},
    {/* numberOfCopies */ 454, 2},
    {/* behaviorOverride */ 456, 2},
    {/* timeStamp */ 458, 4},
    {/* month */ 462, 0},
    {/* viewerPreferences */ 462, 2},
    {/* scriptModel */ 464, 2},
    {/* decimal */ 466, 6},
    {/* subform */ 472, 31},
    {/* select */ 503, 2},
    {/* window */ 505, 2},
    {/* localeSet */ 507, 2},
    {/* handler */ 509, 4},
    {/* hostPseudoModel */ 513, 11},
    {/* presence */ 524, 2},
    {/* record */ 526, 2},
    {/* embed */ 528, 2},
    {/* version */ 530, 2},
    {/* command */ 532, 3},
    {/* copies */ 535, 2},
    {/* staple */ 537, 3},
    {/* submitFormat */ 540, 3},
    {/* boolean */ 543, 4},
    {/* message */ 547, 4},
    {/* output */ 551, 2},
    {/* psMap */ 553, 0},
    {/* excludeNS */ 553, 2},
    {/* assist */ 555, 3},
    {/* picture */ 558, 6},
    {/* traversal */ 564, 2},
    {/* silentPrint */ 566, 2},
    {/* webClient */ 568, 3},
    {/* layoutPseudoModel */ 571, 1},
    {/* producer */ 572, 2},
    {/* corner */ 574, 8},
    {/* msgId */ 582, 2},
    {/* color */ 584, 4},
    {/* keep */ 588, 5},
    {/* query */ 593, 3},
    {/* insert */ 596, 2},
    {/* imageEdit */ 598, 3},
    {/* validate */ 601, 7},
    {/* digestMethods */ 608, 3},
    {/* numberPatterns */ 611, 0},
    {/* pageSet */ 611, 4},
    {/* integer */ 615, 4},
    {/* soapAddress */ 619, 2},
    {/* equate */ 621, 5},
    {/* formFieldFilling */ 626, 2},
    {/* pageRange */ 628, 2},
    {/* update */ 630, 2},
    {/* connectString */ 632, 2},
    {/* mode */ 634, 4},
    {/* layout */ 638, 2},
    {/* #xml */ 640, 1},
    {/* xsdConnection */ 641, 1},
    {/* traverse */ 642, 4},
    {/* encodings */ 646, 3},
    {/* template */ 649, 2},
    {/* acrobat */ 651, 2},
    {/* validationMessaging */ 653, 2},
    {/* signing */ 655, 3},
    {/* dataWindow */ 658, 4},
    {/* script */ 662, 10},
    {/* addViewerPreferences */ 672, 2},
    {/* alwaysEmbed */ 674, 4},
    {/* passwordEdit */ 678, 4},
    {/* numericEdit */ 682, 3},
    {/* encryptionMethod */ 685, 2},
    {/* change */ 687, 2},
    {/* pageArea */ 689, 8},
    {/* submitUrl */ 697, 3},
    {/* oids */ 700, 3},
    {/* signature */ 703, 2},
    {/* ADBE_JSConsole */ 705, 2},
    {/* caption */ 707, 5},
    {/* relevant */ 712, 4},
    {/* flipLabel */ 716, 2},
    {/* exData */ 718, 8},
    {/* dayNames */ 726, 1},
    {/* soapAction */ 727, 2},
    {/* defaultTypeface */ 729, 3},
    {/* manifest */ 732, 4},
    {/* overflow */ 736, 5},
    {/* linear */ 741, 3},
    {/* currencySymbol */ 744, 1},
    {/* delete */ 745, 2},
    {/* deltas */ 747, 0},
    {/* digestMethod */ 747, 2},
    {/* instanceManager */ 749, 3},
    {/* equateRange */ 752, 5},
    {/* medium */ 757, 7},
    {/* textEdit */ 764, 6},
    {/* templateCache */ 770, 3},
    {/* compressObjectStream */ 773, 2},
    {/* dataValue */ 775, 5},
    {/* accessibleContent */ 780, 2},
    {/* nodeList */ 782, 0},
    {/* includeXDPContent */ 782, 2},
    {/* xmlConnection */ 784, 1},
    {/* validateApprovalSignatures */ 785, 2},
    {/* signData */ 787, 5},
    {/* packets */ 792, 2},
    {/* datePattern */ 794, 1},
    {/* duplexOption */ 795, 2},
    {/* base */ 797, 2},
    {/* bind */ 799, 6},
    {/* compression */ 805, 2},
    {/* user */ 807, 2},
    {/* rectangle */ 809, 3},
    {/* effectiveOutputPolicy */ 812, 4},
    {/* ADBE_JSDebugger */ 816, 2},
    {/* acrobat7 */ 818, 2},
    {/* interactive */ 820, 2},
    {/* locale */ 822, 2},
    {/* currentPage */ 824, 2},
    {/* data */ 826, 2},
    {/* date */ 828, 4},
    {/* desc */ 832, 2},
    {/* encrypt */ 834, 5},
    {/* draw */ 839, 20},
    {/* encryption */ 859, 2},
    {/* meridiemNames */ 861, 0},
    {/* messaging */ 861, 2},
    {/* speak */ 863, 4},
    {/* dataGroup */ 867, 0},
    {/* common */ 867, 2},
    {/* #text */ 869, 1},
    {/* paginationOverride */ 870, 2},
    {/* reasons */ 872, 3},
    {/* signatureProperties */ 875, 2},
    {/* threshold */ 877, 2},
    {/* appearanceFilter */ 879, 4},
    {/* fill */ 883, 3},
    {/* font */ 886, 17},
    {/* form */ 903, 1},
    {/* mediumInfo */ 904, 2},
    {/* certificate */ 906, 2},
    {/* password */ 908, 2},
    {/* runScripts */ 910, 2},
    {/* trace */ 912, 2},
    {/* float */ 914, 4},
    {/* renderPolicy */ 918, 2},
    {/* logPseudoModel */ 920, 0},
    {/* destination */ 920, 2},
    {/* value */ 922, 4},
    {/* bookend */ 926, 4},
    {/* exObject */ 930, 6},
    {/* openAction */ 936, 2},
    {/* neverEmbed */ 938, 4},
    {/* bindItems */ 942, 3},
    {/* calculate */ 945, 3},
    {/* print */ 948, 2},
    {/* extras */ 950, 3},
    {/* proto */ 953, 0},
    {/* dSigData */ 953, 0},
    {/* creator */ 953, 2},
    {/* connect */ 955, 7},
    {/* permissions */ 962, 2},
    {/* connectionSet */ 964, 0},
    {/* submit */ 964, 7},
    {/* range */ 971, 2},
    {/* linearized */ 973, 2},
    {/* packet */ 975, 1},
    {/* rootElement */ 976, 2},
    {/* plaintextMetadata */ 978, 4},
    {/* numberSymbols */ 982, 0},
    {/* printHighQuality */ 982, 2},
    {/* driver */ 984, 2},
    {/* incrementalLoad */ 986, 4},
    {/* subjectDN */ 990, 1},
    {/* compressLogicalStructure */ 991, 2},
    {/* incrementalMerge */ 993, 2},
    {/* radial */ 995, 3},
    {/* variables */ 998, 2},
    {/* timePatterns */ 1000, 0},
    {/* effectiveInputPolicy */ 1000, 4},
    {/* nameAttr */ 1004, 4},
    {/* conformance */ 1008, 2},
    {/* transform */ 1010, 3},
    {/* lockDocument */ 1013, 4},
    {/* breakAfter */ 1017, 7},
    {/* line */ 1024, 4},
    {/* list */ 1028, 1},
    {/* source */ 1029, 3},
    {/* occur */ 1032, 5},
    {/* pickTrayByPDFSize */ 1037, 2},
    {/* monthNames */ 1039, 1},
    {/* severity */ 1040, 4},
    {/* groupParent */ 1044, 2},
    {/* documentAssembly */ 1046, 2},
    {/* numberSymbol */ 1048, 1},
    {/* tagged */ 1049, 2},
    {/* items */ 1051, 5},
    {/* 312=Object */ 1056, 1},
    {/* 313=ListDuplicate */ 1057, 1},
    {/* 314=Placeholder1 */ 1058, 0},
    {/* 315=Tree */ 1058, 8},
    {/* 316=Node  */ 1066, 6},
    {/* 317=Placeholder2 */ 1072, 0},
    {/* 318=Placeholder3 */ 1072, 0},
    {/* 319=Model */ 1072, 2},
    {/* 320=Placeholder4 */ 1074, 2}};

static_assert(static_cast<int>(XFA_Element::Object) == 312, "312");
static_assert(static_cast<int>(XFA_Element::ListDuplicate) == 313, "313");
static_assert(static_cast<int>(XFA_Element::Placeholder1) == 314, "314");
static_assert(static_cast<int>(XFA_Element::Tree) == 315, "315");
static_assert(static_cast<int>(XFA_Element::Node) == 316, "316");
static_assert(static_cast<int>(XFA_Element::Placeholder2) == 317, "317");
static_assert(static_cast<int>(XFA_Element::Placeholder3) == 318, "318");
static_assert(static_cast<int>(XFA_Element::Model) == 319, "319");
static_assert(static_cast<int>(XFA_Element::Placeholder4) == 320, "320");

struct ElementAttributeRecord {
  uint32_t uHash;  // Hashed as wide string.
  XFA_Attribute attribute;
  XFA_ScriptType eValueType;
  XFA_ATTRIBUTE_CALLBACK callback;
};

#undef ATTR
#define ATTR(a, b, c, d, e) a, d, e, reinterpret_cast<XFA_ATTRIBUTE_CALLBACK>(c)

const ElementAttributeRecord g_ElementAttributeTable[] = {
    /* ps */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* to */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* ui */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* recordSet */
    {ATTR(0xb3543a6,
          "max",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Max,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x45a6daf8,
          "eofAction",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::EofAction,
          XFA_ScriptType::Basic)},
    {ATTR(0x5ec958c0,
          "cursorType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CursorType,
          XFA_ScriptType::Basic)},
    {ATTR(0x79975f2b,
          "lockType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LockType,
          XFA_ScriptType::Basic)},
    {ATTR(0xa5340ff5,
          "bofAction",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BofAction,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc5762157,
          "cursorLocation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CursorLocation,
          XFA_ScriptType::Basic)},

    /* subsetBelow */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* subformSet */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1ee2d24d,
          "instanceIndex",
          &CJX_SubformSet::instanceIndex,
          XFA_Attribute::InstanceIndex,
          XFA_ScriptType::Basic)},
    {ATTR(0x8c99377e,
          "relation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relation,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* adobeExtensionLevel */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* typeface */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* break */
    {ATTR(0x3106c3a,
          "beforeTarget",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BeforeTarget,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x13a08bdb,
          "overflowTarget",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::OverflowTarget,
          XFA_ScriptType::Basic)},
    {ATTR(0x169134a1,
          "overflowLeader",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::OverflowLeader,
          XFA_ScriptType::Basic)},
    {ATTR(0x20914367,
          "overflowTrailer",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::OverflowTrailer,
          XFA_ScriptType::Basic)},
    {ATTR(0x453eaf38,
          "startNew",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::StartNew,
          XFA_ScriptType::Basic)},
    {ATTR(0x64110ab5,
          "bookendTrailer",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BookendTrailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xb6b44172,
          "after",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::After,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc3c1442f,
          "bookendLeader",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BookendLeader,
          XFA_ScriptType::Basic)},
    {ATTR(0xcb150479,
          "afterTarget",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AfterTarget,
          XFA_ScriptType::Basic)},
    {ATTR(0xf4ffce73,
          "before",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Before,
          XFA_ScriptType::Basic)},

    /* fontInfo */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* numberPattern */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* dynamicRender */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptSomDefaultValue_Read,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* printScaling */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* checkButton */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x47cfa43a,
          "allowNeutral",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AllowNeutral,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2fd80b,
          "mark",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Mark,
          XFA_ScriptType::Basic)},
    {ATTR(0x8ed182d1,
          "shape",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Shape,
          XFA_ScriptType::Basic)},
    {ATTR(0xa686975b,
          "size",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Size,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* datePatterns */

    /* sourceSet */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* amd */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* arc */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5c054755,
          "startAngle",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::StartAngle,
          XFA_ScriptType::Basic)},
    {ATTR(0x74788f8b,
          "sweepAngle",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::SweepAngle,
          XFA_ScriptType::Basic)},
    {ATTR(0x9d833d75,
          "circular",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Circular,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          "hand",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* day */

    /* era */

    /* jog */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* log */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* map */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xb0e5485d,
          "bind",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Bind,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xcd7f7b54,
          "from",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::From,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* mdp */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e29d794,
          "signatureType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::SignatureType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe11a2cbc,
          "permissions",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Permissions,
          XFA_ScriptType::Basic)},

    /* breakBefore */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x453eaf38,
          "startNew",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::StartNew,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          "trailer",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xa6118c89,
          "targetType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TargetType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          "leader",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* oid */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* pcl */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pdf */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* ref */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* uri */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xdc */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xdp */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xfa */
    {ATTR(0x2d574d58,
          "this",
          &CJX_Xfa::thisValue,
          XFA_Attribute::ThisValue,
          XFA_ScriptType::Object)},
    {ATTR(0x4fdc3454,
          "timeStamp",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TimeStamp,
          XFA_ScriptType::Basic)},
    {ATTR(0xb598a1f7,
          "uuid",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Uuid,
          XFA_ScriptType::Basic)},

    /* xsl */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* zpl */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* cache */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* margin */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xcfea02e,
          "leftInset",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LeftInset,
          XFA_ScriptType::Basic)},
    {ATTR(0x1356caf8,
          "bottomInset",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BottomInset,
          XFA_ScriptType::Basic)},
    {ATTR(0x25764436,
          "topInset",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TopInset,
          XFA_ScriptType::Basic)},
    {ATTR(0x8a692521,
          "rightInset",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RightInset,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* keyUsage */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1e459b8f,
          "nonRepudiation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::NonRepudiation,
          XFA_ScriptType::Basic)},
    {ATTR(0x2bb3f470,
          "encipherOnly",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::EncipherOnly,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x5f760b50,
          "digitalSignature",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DigitalSignature,
          XFA_ScriptType::Basic)},
    {ATTR(0x69aa2292,
          "crlSign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CrlSign,
          XFA_ScriptType::Basic)},
    {ATTR(0x98fd4d81,
          "keyAgreement",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::KeyAgreement,
          XFA_ScriptType::Basic)},
    {ATTR(0xa66404cb,
          "keyEncipherment",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::KeyEncipherment,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xca5dc27c,
          "dataEncipherment",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataEncipherment,
          XFA_ScriptType::Basic)},
    {ATTR(0xe8f118a8,
          "keyCertSign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::KeyCertSign,
          XFA_ScriptType::Basic)},
    {ATTR(0xfea53ec6,
          "decipherOnly",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DecipherOnly,
          XFA_ScriptType::Basic)},

    /* exclude */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* choiceList */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8b90e1f2,
          "open",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Open,
          XFA_ScriptType::Basic)},
    {ATTR(0x957fa006,
          "commitOn",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CommitOn,
          XFA_ScriptType::Basic)},
    {ATTR(0xb12128b7,
          "textEntry",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TextEntry,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* level */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* labelPrinter */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* calendarSymbols */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* para */
    {ATTR(0x2282c73,
          "hAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x8d4f1c7,
          "textIndent",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TextIndent,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2a82d99c,
          "marginRight",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MarginRight,
          XFA_ScriptType::Basic)},
    {ATTR(0x534729c9,
          "marginLeft",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MarginLeft,
          XFA_ScriptType::Basic)},
    {ATTR(0x5739d1ff,
          "radixOffset",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RadixOffset,
          XFA_ScriptType::Basic)},
    {ATTR(0x577682ac,
          "preserve",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Preserve,
          XFA_ScriptType::Basic)},
    {ATTR(0x731e0665,
          "spaceBelow",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::SpaceBelow,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          "vAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x836d4d7c,
          "tabDefault",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TabDefault,
          XFA_ScriptType::Basic)},
    {ATTR(0x8fa01790,
          "tabStops",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TabStops,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd4b01921,
          "lineHeight",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LineHeight,
          XFA_ScriptType::Basic)},
    {ATTR(0xe18b5659,
          "spaceAbove",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::SpaceAbove,
          XFA_ScriptType::Basic)},

    /* part */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pdfa */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* filter */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd861f8af,
          "addRevocationInfo",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AddRevocationInfo,
          XFA_ScriptType::Basic)},

    /* present */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pagination */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* encoding */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* event */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6128d8d,
          "activity",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Activity,
          XFA_ScriptType::Basic)},

    /* whitespace */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* defaultUi */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* dataModel */

    /* barcode */
    {ATTR(0x43e349b,
          "dataRowCount",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataRowCount,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x28e17e91,
          "dataPrep",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataPrep,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x3650557e,
          "textLocation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TextLocation,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b582286,
          "moduleWidth",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ModuleWidth,
          XFA_ScriptType::Basic)},
    {ATTR(0x52666f1c,
          "printCheckDigit",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::PrintCheckDigit,
          XFA_ScriptType::Basic)},
    {ATTR(0x5404d6df,
          "moduleHeight",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ModuleHeight,
          XFA_ScriptType::Basic)},
    {ATTR(0x5ab23b6c,
          "startChar",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::StartChar,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c732a66,
          "truncate",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Truncate,
          XFA_ScriptType::Basic)},
    {ATTR(0x8d181d61,
          "wideNarrowRatio",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::WideNarrowRatio,
          XFA_ScriptType::Basic)},
    {ATTR(0x99800d7a,
          "errorCorrectionLevel",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ErrorCorrectionLevel,
          XFA_ScriptType::Basic)},
    {ATTR(0x9a63da3d,
          "upsMode",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::UpsMode,
          XFA_ScriptType::Basic)},
    {ATTR(0xaf754613,
          "checksum",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Checksum,
          XFA_ScriptType::Basic)},
    {ATTR(0xb045fbc5,
          "charEncoding",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CharEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc035c6b1,
          "dataColumnCount",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataColumnCount,
          XFA_ScriptType::Basic)},
    {ATTR(0xd3c84d25,
          "rowColumnRatio",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RowColumnRatio,
          XFA_ScriptType::Basic)},
    {ATTR(0xd57c513c,
          "dataLength",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataLength,
          XFA_ScriptType::Basic)},
    {ATTR(0xf575ca75,
          "endChar",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::EndChar,
          XFA_ScriptType::Basic)},

    /* timePattern */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* batchOutput */
    {ATTR(0x28dee6e9,
          "format",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Format,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* enforce */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* currencySymbols */

    /* addSilentPrint */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* rename */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* operation */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x60d4c8b1,
          "output",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Output,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6a39990,
          "input",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Input,
          XFA_ScriptType::Basic)},

    /* typefaces */

    /* subjectDNs */
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},

    /* issuers */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* signaturePseudoModel */

    /* wsdlConnection */
    {ATTR(0x2b5df51e,
          "dataDescription",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataDescription,
          XFA_ScriptType::Basic)},

    /* debug */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* delta */
    {ATTR(0x6c0d9600,
          "currentValue",
          &CJX_Delta::currentValue,
          XFA_Attribute::CurrentValue,
          XFA_ScriptType::Basic)},
    {ATTR(0x942643f0,
          "savedValue",
          &CJX_Delta::savedValue,
          XFA_Attribute::SavedValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Delta::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},

    /* eraNames */

    /* modifyAnnots */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* startNode */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* button */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd4cc53f8,
          "highlight",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Highlight,
          XFA_ScriptType::Basic)},

    /* format */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* border */
    {ATTR(0x5518c25,
          "break",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Break,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          "hand",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* area */
    {ATTR(0x78,
          "x",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          "y",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1059ec18,
          "level",
          &CJX_Object::ScriptAttributeInteger,
          XFA_Attribute::Level,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          "colSpan",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* hyphenation */
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f105f72,
          "wordCharacterCount",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::WordCharacterCount,
          XFA_ScriptType::Basic)},
    {ATTR(0x3d123c26,
          "hyphenate",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Hyphenate,
          XFA_ScriptType::Basic)},
    {ATTR(0x66539c48,
          "excludeInitialCap",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ExcludeInitialCap,
          XFA_ScriptType::Basic)},
    {ATTR(0x6a95c976,
          "pushCharacterCount",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::PushCharacterCount,
          XFA_ScriptType::Basic)},
    {ATTR(0x982bd892,
          "remainCharacterCount",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RemainCharacterCount,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe5c96d6a,
          "excludeAllCaps",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ExcludeAllCaps,
          XFA_ScriptType::Basic)},

    /* text */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8af2e657,
          "maxChars",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxChars,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Text::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Text::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* time */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Time::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Time::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* type */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* overprint */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* certificates */
    {ATTR(0xc080cd3,
          "url",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Url,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa6710262,
          "credentialServerPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CredentialServerPolicy,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2ba0923,
          "urlPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::UrlPolicy,
          XFA_ScriptType::Basic)},

    /* encryptionMethods */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* setProperty */
    {ATTR(0x47d03490,
          "connection",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},

    /* printerName */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* startPage */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pageOffset */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* dateTime */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_DateTime::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_DateTime::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* comb */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x78bff531,
          "numberOfCells",
          &CJX_Comb::numberOfCells,
          XFA_Attribute::NumberOfCells,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* pattern */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* ifEmpty */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* suppressBanner */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* outputBin */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* field */
    {ATTR(0x68,
          "h",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          "w",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          "x",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          "y",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          "hAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1abbd7e0,
          "dataNode",
          &CJX_Object::ScriptSomDataNode,
          XFA_Attribute::DataNode,
          XFA_ScriptType::Object)},
    {ATTR(0x25839852,
          "access",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Access,
          XFA_ScriptType::Basic)},
    {ATTR(0x2ee7678f,
          "rotate",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Rotate,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b1ddd06,
          "fillColor",
          &CJX_Field::fillColor,
          XFA_Attribute::FillColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x54c399e3,
          "formattedValue",
          &CJX_Field::formattedValue,
          XFA_Attribute::FormattedValue,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a3b375d,
          "borderColor",
          &CJX_Field::borderColor,
          XFA_Attribute::BorderColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x5e936ed6,
          "fontColor",
          &CJX_Field::fontColor,
          XFA_Attribute::FontColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x6826c408,
          "parentSubform",
          &CJX_Field::parentSubform,
          XFA_Attribute::ParentSubform,
          XFA_ScriptType::Basic)},
    {ATTR(0x79b67434,
          "mandatoryMessage",
          &CJX_Field::mandatoryMessage,
          XFA_Attribute::MandatoryMessage,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          "vAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          "maxH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          "maxW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          "minH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          "minW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x85fd6faf,
          "mandatory",
          &CJX_Field::mandatory,
          XFA_Attribute::Mandatory,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0x964fb42e,
          "formatMessage",
          &CJX_Field::formatMessage,
          XFA_Attribute::FormatMessage,
          XFA_ScriptType::Basic)},
    {ATTR(0xa03cf627,
          "rawValue",
          &CJX_Field::rawValue,
          XFA_Attribute::RawValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Field::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xa60dd202,
          "length",
          &CJX_Object::ScriptFieldLength,
          XFA_Attribute::Length,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          "colSpan",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc8fa350,
          "locale",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Locale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          "anchorType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},
    {ATTR(0xc4fed09b,
          "accessKey",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AccessKey,
          XFA_ScriptType::Basic)},
    {ATTR(0xcabfa3d0,
          "validationMessage",
          &CJX_Field::validationMessage,
          XFA_Attribute::ValidationMessage,
          XFA_ScriptType::Basic)},
    {ATTR(0xdcecd663,
          "editValue",
          &CJX_Field::editValue,
          XFA_Attribute::EditValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xe07e5061,
          "selectedIndex",
          &CJX_Field::selectedIndex,
          XFA_Attribute::SelectedIndex,
          XFA_ScriptType::Basic)},
    {ATTR(0xf65e34be,
          "borderWidth",
          &CJX_Field::borderWidth,
          XFA_Attribute::BorderWidth,
          XFA_ScriptType::Basic)},

    /* agent */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* outputXSL */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* adjustData */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* autoSave */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* contentArea */
    {ATTR(0x68,
          "h",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          "w",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          "x",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          "y",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* eventPseudoModel */
    {ATTR(0xd843798,
          "fullText",
          &CJX_EventPseudoModel::fullText,
          XFA_Attribute::FullText,
          XFA_ScriptType::Basic)},
    {ATTR(0x1b6d1cf5,
          "reenter",
          &CJX_EventPseudoModel::reenter,
          XFA_Attribute::Reenter,
          XFA_ScriptType::Basic)},
    {ATTR(0x1e6ffa9a,
          "prevContentType",
          &CJX_EventPseudoModel::prevContentType,
          XFA_Attribute::PrevContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x25a3c206,
          "soapFaultString",
          &CJX_EventPseudoModel::soapFaultString,
          XFA_Attribute::SoapFaultString,
          XFA_ScriptType::Basic)},
    {ATTR(0x2e00c007,
          "newContentType",
          &CJX_EventPseudoModel::newContentType,
          XFA_Attribute::NewContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x4570500f,
          "modifier",
          &CJX_EventPseudoModel::modifier,
          XFA_Attribute::Modifier,
          XFA_ScriptType::Basic)},
    {ATTR(0x50e2e33b,
          "selEnd",
          &CJX_EventPseudoModel::selEnd,
          XFA_Attribute::SelEnd,
          XFA_ScriptType::Basic)},
    {ATTR(0x57de87c2,
          "prevText",
          &CJX_EventPseudoModel::prevText,
          XFA_Attribute::PrevText,
          XFA_ScriptType::Basic)},
    {ATTR(0x6ea04e0a,
          "soapFaultCode",
          &CJX_EventPseudoModel::soapFaultCode,
          XFA_Attribute::SoapFaultCode,
          XFA_ScriptType::Basic)},
    {ATTR(0x6f6556cf,
          "newText",
          &CJX_EventPseudoModel::newText,
          XFA_Attribute::NewText,
          XFA_ScriptType::Basic)},
    {ATTR(0x891f4606,
          "change",
          &CJX_EventPseudoModel::change,
          XFA_Attribute::Change,
          XFA_ScriptType::Basic)},
    {ATTR(0x8fa3c19e,
          "shift",
          &CJX_EventPseudoModel::shift,
          XFA_Attribute::Shift,
          XFA_ScriptType::Basic)},
    {ATTR(0xa9d9b2e1,
          "keyDown",
          &CJX_EventPseudoModel::keyDown,
          XFA_Attribute::keyDown,
          XFA_ScriptType::Basic)},
    {ATTR(0xbfc89db2,
          "selStart",
          &CJX_EventPseudoModel::selStart,
          XFA_Attribute::selStart,
          XFA_ScriptType::Basic)},
    {ATTR(0xc32a5812,
          "commitKey",
          &CJX_EventPseudoModel::commitKey,
          XFA_Attribute::CommitKey,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_EventPseudoModel::target,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xd8624e04,
          "cancelAction",
          &CJX_EventPseudoModel::cancelAction,
          XFA_Attribute::cancelAction,
          XFA_ScriptType::Basic)},

    /* wsdlAddress */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* solid */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* dateTimeSymbols */

    /* encryptionLevel */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* edge */
    {ATTR(0xa2e3514,
          "cap",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Cap,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5392ea58,
          "stroke",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Stroke,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x94446dcc,
          "thickness",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Thickness,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* stipple */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1ec8ab2c,
          "rate",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Rate,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* attributes */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* versionControl */
    {ATTR(0x7b29630a,
          "sourceBelow",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::SourceBelow,
          XFA_ScriptType::Basic)},
    {ATTR(0x8fc36c0a,
          "outputBelow",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::OutputBelow,
          XFA_ScriptType::Basic)},
    {ATTR(0xe996b2fe,
          "sourceAbove",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::SourceAbove,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* meridiem */

    /* exclGroup */
    {ATTR(0x68,
          "h",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          "w",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          "x",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          "y",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          "hAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xf23332f,
          "errorText",
          &CJX_Object::ScriptExclGroupErrorText,
          XFA_Attribute::ErrorText,
          XFA_ScriptType::Basic)},
    {ATTR(0x1abbd7e0,
          "dataNode",
          &CJX_Object::ScriptSomDataNode,
          XFA_Attribute::DataNode,
          XFA_ScriptType::Object)},
    {ATTR(0x25839852,
          "access",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Access,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b1ddd06,
          "fillColor",
          &CJX_ExclGroup::fillColor,
          XFA_Attribute::FillColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a3b375d,
          "borderColor",
          &CJX_ExclGroup::borderColor,
          XFA_Attribute::BorderColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x79b67434,
          "mandatoryMessage",
          &CJX_ExclGroup::mandatoryMessage,
          XFA_Attribute::MandatoryMessage,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          "vAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          "maxH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          "maxW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          "minH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          "minW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7e7e845e,
          "layout",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Layout,
          XFA_ScriptType::Basic)},
    {ATTR(0x846599f8,
          "transient",
          &CJX_ExclGroup::transient,
          XFA_Attribute::Transient,
          XFA_ScriptType::Basic)},
    {ATTR(0x85fd6faf,
          "mandatory",
          &CJX_ExclGroup::mandatory,
          XFA_Attribute::Mandatory,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xa03cf627,
          "rawValue",
          &CJX_ExclGroup::rawValue,
          XFA_Attribute::RawValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_ExclGroup::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          "colSpan",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          "anchorType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},
    {ATTR(0xc4fed09b,
          "accessKey",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AccessKey,
          XFA_ScriptType::Basic)},
    {ATTR(0xcabfa3d0,
          "validationMessage",
          &CJX_ExclGroup::validationMessage,
          XFA_Attribute::ValidationMessage,
          XFA_ScriptType::Basic)},
    {ATTR(0xf65e34be,
          "borderWidth",
          &CJX_ExclGroup::borderWidth,
          XFA_Attribute::BorderWidth,
          XFA_ScriptType::Basic)},

    /* toolTip */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* compress */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xeda9017a,
          "scope",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Scope,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* reason */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* execute */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x47d03490,
          "connection",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0x6cfa828a,
          "runAt",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RunAt,
          XFA_ScriptType::Basic)},
    {ATTR(0xa1b0d2f5,
          "executeType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ExecuteType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* contentCopy */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* dateTimeEdit */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          "hScrollPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* config */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* image */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          "contentType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x54fa722c,
          "transferEncoding",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TransferEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Image::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd171b240,
          "aspect",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Aspect,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Image::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xdb55fec5,
          "href",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Href,
          XFA_ScriptType::Basic)},

    /* #xHTML */
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* numberOfCopies */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* behaviorOverride */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* timeStamp */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x7f6fd3d7,
          "server",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Server,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* month */

    /* viewerPreferences */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* scriptModel */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* decimal */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x4b8bc840,
          "fracDigits",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::FracDigits,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Decimal::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Decimal::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xde7f92ba,
          "leadDigits",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LeadDigits,
          XFA_ScriptType::Basic)},

    /* subform */
    {ATTR(0x68,
          "h",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          "w",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          "x",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          "y",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          "hAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1414d431,
          "allowMacro",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AllowMacro,
          XFA_ScriptType::Basic)},
    {ATTR(0x1517dfa1,
          "columnWidths",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ColumnWidths,
          XFA_ScriptType::Basic)},
    {ATTR(0x1abbd7e0,
          "dataNode",
          &CJX_Object::ScriptSomDataNode,
          XFA_Attribute::DataNode,
          XFA_ScriptType::Object)},
    {ATTR(0x1ee2d24d,
          "instanceIndex",
          &CJX_Subform::instanceIndex,
          XFA_Attribute::InstanceIndex,
          XFA_ScriptType::Basic)},
    {ATTR(0x25839852,
          "access",
          &CJX_Object::ScriptAttributeString,

          XFA_Attribute::Access,
          XFA_ScriptType::Basic)},
    {ATTR(0x3b1ddd06,
          "fillColor",
          &CJX_Object::ScriptSomFillColor,
          XFA_Attribute::FillColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a3b375d,
          "borderColor",
          &CJX_Object::ScriptSomBorderColor,
          XFA_Attribute::BorderColor,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          "vAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          "maxH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          "maxW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          "minH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          "minW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7e7e845e,
          "layout",
          &CJX_Subform::layout,
          XFA_Attribute::Layout,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0x9cc17d75,
          "mergeMode",
          &CJX_Object::ScriptAttributeString,

          XFA_Attribute::MergeMode,
          XFA_ScriptType::Basic)},
    {ATTR(0x9f3e9510,
          "instanceManager",
          &CJX_Object::ScriptSubformInstanceManager,
          XFA_Attribute::InstanceManager,
          XFA_ScriptType::Object)},
    {ATTR(0xac06e2b0,
          "colSpan",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc8fa350,
          "locale",
          &CJX_Subform::locale,
          XFA_Attribute::Locale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          "anchorType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},
    {ATTR(0xcabfa3d0,
          "validationMessage",
          &CJX_Subform::validationMessage,
          XFA_Attribute::ValidationMessage,
          XFA_ScriptType::Basic)},
    {ATTR(0xe4c3a5e5,
          "restoreState",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RestoreState,
          XFA_ScriptType::Basic)},
    {ATTR(0xeda9017a,
          "scope",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Scope,
          XFA_ScriptType::Basic)},
    {ATTR(0xf65e34be,
          "borderWidth",
          &CJX_Object::ScriptSomBorderWidth,
          XFA_Attribute::BorderWidth,
          XFA_ScriptType::Basic)},

    /* select */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* window */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* localeSet */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* handler */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a50e9e6,
          "version",
          &CJX_Handler::version,
          XFA_Attribute::Version,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* hostPseudoModel */
    {ATTR(0x31b19c1,
          "name",
          &CJX_HostPseudoModel::name,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0x66c1ae9,
          "validationsEnabled",
          &CJX_HostPseudoModel::validationsEnabled,
          XFA_Attribute::ValidationsEnabled,
          XFA_ScriptType::Basic)},
    {ATTR(0x14d04502,
          "title",
          &CJX_HostPseudoModel::title,
          XFA_Attribute::Title,
          XFA_ScriptType::Basic)},
    {ATTR(0x392ae445,
          "platform",
          &CJX_HostPseudoModel::platform,
          XFA_Attribute::Platform,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a50e9e6,
          "version",
          &CJX_HostPseudoModel::version,
          XFA_Attribute::Version,
          XFA_ScriptType::Basic)},
    {ATTR(0x66cb1eed,
          "variation",
          &CJX_HostPseudoModel::variation,
          XFA_Attribute::Variation,
          XFA_ScriptType::Basic)},
    {ATTR(0x7717cbc4,
          "language",
          &CJX_HostPseudoModel::language,
          XFA_Attribute::Language,
          XFA_ScriptType::Basic)},
    {ATTR(0x86698963,
          "appType",
          &CJX_HostPseudoModel::appType,
          XFA_Attribute::AppType,
          XFA_ScriptType::Basic)},
    {ATTR(0x94ff9e8d,
          "calculationsEnabled",
          &CJX_HostPseudoModel::calculationsEnabled,
          XFA_Attribute::CalculationsEnabled,
          XFA_ScriptType::Basic)},
    {ATTR(0xbcd44940,
          "currentPage",
          &CJX_HostPseudoModel::currentPage,
          XFA_Attribute::CurrentPage,
          XFA_ScriptType::Basic)},
    {ATTR(0xd592b920,
          "numPages",
          &CJX_HostPseudoModel::numPages,
          XFA_Attribute::NumPages,
          XFA_ScriptType::Basic)},

    /* presence */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* record */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* embed */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* version */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* command */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x24d85167,
          "timeout",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Timeout,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* copies */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* staple */
    {ATTR(0x7d9fd7c5,
          "mode",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Mode,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* submitFormat */
    {ATTR(0x7d9fd7c5,
          "mode",
          &CJX_Object::ScriptSubmitFormatMode,
          XFA_Attribute::Mode,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* boolean */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Boolean::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Boolean::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* message */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* output */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* psMap */

    /* excludeNS */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* assist */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2038c9b2,
          "role",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Role,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* picture */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Picture::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Picture::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* traversal */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* silentPrint */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* webClient */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* layoutPseudoModel */
    {ATTR(0xfcef86b5,
          "ready",
          &CJX_LayoutPseudoModel::ready,
          XFA_Attribute::Ready,
          XFA_ScriptType::Basic)},

    /* producer */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* corner */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5392ea58,
          "stroke",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Stroke,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x7b95e661,
          "inverted",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Inverted,
          XFA_ScriptType::Basic)},
    {ATTR(0x94446dcc,
          "thickness",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Thickness,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe8dddf50,
          "join",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Join,
          XFA_ScriptType::Basic)},
    {ATTR(0xe948b9a8,
          "radius",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Radius,
          XFA_ScriptType::Basic)},

    /* msgId */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* color */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xabfa6c4f,
          "cSpace",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CSpace,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* keep */
    {ATTR(0x3848b3f,
          "next",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Next,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x6a3405dd,
          "previous",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Previous,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b59543,
          "intact",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Intact,
          XFA_ScriptType::Basic)},

    /* query */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x268b7ec1,
          "commandType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CommandType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* insert */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* imageEdit */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbde9abda,
          "data",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Data,
          XFA_ScriptType::Basic)},

    /* validate */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x5b707a35,
          "scriptTest",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ScriptTest,
          XFA_ScriptType::Basic)},
    {ATTR(0x6b6ddcfb,
          "nullTest",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::NullTest,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xe64b1129,
          "formatTest",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::FormatTest,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* digestMethods */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* numberPatterns */

    /* pageSet */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8c99377e,
          "relation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relation,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* integer */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Integer::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Integer::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* soapAddress */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* equate */
    {ATTR(0x25363,
          "to",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::To,
          XFA_ScriptType::Basic)},
    {ATTR(0x66642f8f,
          "force",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Force,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xcd7f7b54,
          "from",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::From,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* formFieldFilling */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pageRange */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* update */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* connectString */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* mode */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* layout */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* #xml */
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* xsdConnection */
    {ATTR(0x2b5df51e,
          "dataDescription",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataDescription,
          XFA_ScriptType::Basic)},

    /* traverse */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x226ca8f1,
          "operation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Operation,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* encodings */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* template */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* acrobat */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* validationMessaging */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* signing */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* dataWindow */
    {ATTR(0xfb67185,
          "recordsBefore",
          &CJX_DataWindow::recordsBefore,
          XFA_Attribute::RecordsBefore,
          XFA_ScriptType::Basic)},
    {ATTR(0x21d5dfcb,
          "currentRecordNumber",
          &CJX_DataWindow::currentRecordNumber,
          XFA_Attribute::CurrentRecordNumber,
          XFA_ScriptType::Basic)},
    {ATTR(0x312af044,
          "recordsAfter",
          &CJX_DataWindow::recordsAfter,
          XFA_Attribute::RecordsAfter,
          XFA_ScriptType::Basic)},
    {ATTR(0x6aab37cb,
          "isDefined",
          &CJX_DataWindow::isDefined,
          XFA_Attribute::IsDefined,
          XFA_ScriptType::Basic)},

    /* script */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          "contentType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x6cfa828a,
          "runAt",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::RunAt,
          XFA_ScriptType::Basic)},
    {ATTR(0xa021b738,
          "stateless",
          &CJX_Script::stateless,
          XFA_Attribute::Stateless,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Script::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xadc4c77b,
          "binding",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Binding,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Script::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* addViewerPreferences */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* alwaysEmbed */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* passwordEdit */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a0cc471,
          "passwordChar",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::PasswordChar,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          "hScrollPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* numericEdit */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          "hScrollPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* encryptionMethod */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* change */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* pageArea */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x14a32d52,
          "pagePosition",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::PagePosition,
          XFA_ScriptType::Basic)},
    {ATTR(0x8340ea66,
          "oddOrEven",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::OddOrEven,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xa85e74f3,
          "initialNumber",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::InitialNumber,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe9ba472,
          "numbered",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Numbered,
          XFA_ScriptType::Basic)},
    {ATTR(0xd70798c2,
          "blankOrNotBlank",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BlankOrNotBlank,
          XFA_ScriptType::Basic)},

    /* submitUrl */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptSomDefaultValue,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* oids */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* signature */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* aDBE_JSConsole */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* caption */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x34ae103c,
          "reserve",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Reserve,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf2009339,
          "placement",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Placement,
          XFA_ScriptType::Basic)},

    /* relevant */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* flipLabel */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* exData */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          "contentType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x54fa722c,
          "transferEncoding",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TransferEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_ExData::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc4547a08,
          "maxLength",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxLength,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptSomDefaultValue,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xdb55fec5,
          "href",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Href,
          XFA_ScriptType::Basic)},

    /* dayNames */
    {ATTR(0x29418bb7,
          "abbr",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Abbr,
          XFA_ScriptType::Basic)},

    /* soapAction */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* defaultTypeface */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf531b059,
          "writingScript",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::WritingScript,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* manifest */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1b8dce3e,
          "action",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Action,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Manifest::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* overflow */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          "trailer",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          "leader",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* linear */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* currencySymbol */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* delete */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* deltas */

    /* digestMethod */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* instanceManager */
    {ATTR(0xb3543a6,
          "max",
          &CJX_InstanceManager::max,
          XFA_Attribute::Max,
          XFA_ScriptType::Basic)},
    {ATTR(0xb356ca4,
          "min",
          &CJX_InstanceManager::min,
          XFA_Attribute::Min,
          XFA_ScriptType::Basic)},
    {ATTR(0x6f544d49,
          "count",
          &CJX_InstanceManager::count,
          XFA_Attribute::Count,
          XFA_ScriptType::Basic)},

    /* equateRange */
    {ATTR(0x25363,
          "to",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::To,
          XFA_ScriptType::Basic)},
    {ATTR(0xa0933954,
          "unicodeRange",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::UnicodeRange,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xcd7f7b54,
          "from",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::From,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* medium */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x4ef3d02c,
          "orientation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Orientation,
          XFA_ScriptType::Basic)},
    {ATTR(0x65e30c67,
          "imagingBBox",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ImagingBBox,
          XFA_ScriptType::Basic)},
    {ATTR(0x9041d4b0,
          "short",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Short,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe349d044,
          "stock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Stock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b4afb0,
          "long",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Long,
          XFA_ScriptType::Basic)},

    /* textEdit */
    {ATTR(0x5ce6195,
          "vScrollPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::VScrollPolicy,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x1ef3a64a,
          "allowRichText",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AllowRichText,
          XFA_ScriptType::Basic)},
    {ATTR(0x5a32e493,
          "multiLine",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MultiLine,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe6f99487,
          "hScrollPolicy",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HScrollPolicy,
          XFA_ScriptType::Basic)},

    /* templateCache */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xd52482e0,
          "maxEntries",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxEntries,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* compressObjectStream */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* dataValue */
    {ATTR(0x42fed1fd,
          "contentType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x8855805f,
          "contains",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Contains,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_DataValue::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_DataValue::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
    {ATTR(0xe372ae97,
          "isNull",
          &CJX_DataValue::isNull,
          XFA_Attribute::IsNull,
          XFA_ScriptType::Basic)},

    /* accessibleContent */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* nodeList */

    /* includeXDPContent */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* xmlConnection */
    {ATTR(0x2b5df51e,
          "dataDescription",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DataDescription,
          XFA_ScriptType::Basic)},

    /* validateApprovalSignatures */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* signData */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x226ca8f1,
          "operation",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Operation,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},

    /* packets */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* datePattern */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* duplexOption */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* base */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* bind */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x42fed1fd,
          "contentType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ContentType,
          XFA_ScriptType::Basic)},
    {ATTR(0x54fa722c,
          "transferEncoding",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TransferEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xf197844d,
          "match",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Match,
          XFA_ScriptType::Basic)},

    /* compression */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* user */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* rectangle */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          "hand",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* effectiveOutputPolicy */
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* aDBE_JSDebugger */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* acrobat7 */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* interactive */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* locale */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* currentPage */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* data */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* date */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Date::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Date::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* desc */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* encrypt */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x28dee6e9,
          "format",
          &CJX_Encrypt::format,
          XFA_Attribute::Format,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* draw */
    {ATTR(0x68,
          "h",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::H,
          XFA_ScriptType::Basic)},
    {ATTR(0x77,
          "w",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::W,
          XFA_ScriptType::Basic)},
    {ATTR(0x78,
          "x",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::X,
          XFA_ScriptType::Basic)},
    {ATTR(0x79,
          "y",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Y,
          XFA_ScriptType::Basic)},
    {ATTR(0x2282c73,
          "hAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::HAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2ee7678f,
          "rotate",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Rotate,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0x7a7cc341,
          "vAlign",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::VAlign,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6ae,
          "maxH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7c2ff6bd,
          "maxW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MaxW,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02356c,
          "minH",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinH,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d02357b,
          "minW",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::MinW,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xa03cf627,
          "rawValue",
          &CJX_Draw::rawValue,
          XFA_Attribute::RawValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Draw::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xac06e2b0,
          "colSpan",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ColSpan,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc8fa350,
          "locale",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Locale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc2bd40fd,
          "anchorType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::AnchorType,
          XFA_ScriptType::Basic)},

    /* encryption */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* meridiemNames */

    /* messaging */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* speak */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x39cdb0a2,
          "priority",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Priority,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xeb511b54,
          "disable",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Disable,
          XFA_ScriptType::Basic)},

    /* dataGroup */

    /* common */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* #text */
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* paginationOverride */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* reasons */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* signatureProperties */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* threshold */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* appearanceFilter */
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* fill */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* font */
    {ATTR(0xcb0ac9,
          "lineThrough",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LineThrough,
          XFA_ScriptType::Basic)},
    {ATTR(0x2c1c7f1,
          "typeface",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Typeface,
          XFA_ScriptType::Basic)},
    {ATTR(0x8c74ae9,
          "fontHorizontalScale",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::FontHorizontalScale,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2cd79033,
          "kerningMode",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::KerningMode,
          XFA_ScriptType::Basic)},
    {ATTR(0x3a0273a6,
          "underline",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Underline,
          XFA_ScriptType::Basic)},
    {ATTR(0x4873c601,
          "baselineShift",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::BaselineShift,
          XFA_ScriptType::Basic)},
    {ATTR(0x4b319767,
          "overlinePeriod",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::OverlinePeriod,
          XFA_ScriptType::Basic)},
    {ATTR(0x79543055,
          "letterSpacing",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LetterSpacing,
          XFA_ScriptType::Basic)},
    {ATTR(0x8ec6204c,
          "lineThroughPeriod",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LineThroughPeriod,
          XFA_ScriptType::Basic)},
    {ATTR(0x907c7719,
          "fontVerticalScale",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::FontVerticalScale,
          XFA_ScriptType::Basic)},
    {ATTR(0xa686975b,
          "size",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Size,
          XFA_ScriptType::Basic)},
    {ATTR(0xb5e49bf2,
          "posture",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Posture,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbd6e1d88,
          "weight",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Weight,
          XFA_ScriptType::Basic)},
    {ATTR(0xbd96a0e9,
          "underlinePeriod",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::UnderlinePeriod,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0ec9fa4,
          "overline",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Overline,
          XFA_ScriptType::Basic)},

    /* form */
    {ATTR(0xaf754613,
          "checksum",
          &CJX_Object::ScriptFormChecksumS,
          XFA_Attribute::Checksum,
          XFA_ScriptType::Basic)},

    /* mediumInfo */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* certificate */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* password */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* runScripts */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* trace */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* float */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Float::defaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Float::value,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},

    /* renderPolicy */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* logPseudoModel */

    /* destination */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* value */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x8e1c2921,
          "relevant",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Relevant,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xea7090a0,
          "override",
          &CJX_Value::override,
          XFA_Attribute::Override,
          XFA_ScriptType::Basic)},

    /* bookend */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          "trailer",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          "leader",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* exObject */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x60a61edd,
          "codeType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CodeType,
          XFA_ScriptType::Basic)},
    {ATTR(0xb373a862,
          "archive",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Archive,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xe1a26b56,
          "codeBase",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::CodeBase,
          XFA_ScriptType::Basic)},
    {ATTR(0xeb091003,
          "classId",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ClassId,
          XFA_ScriptType::Basic)},

    /* openAction */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* neverEmbed */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* bindItems */
    {ATTR(0x47d03490,
          "connection",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0xc39a88bd,
          "labelRef",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::LabelRef,
          XFA_ScriptType::Basic)},
    {ATTR(0xd50f903a,
          "valueRef",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::ValueRef,
          XFA_ScriptType::Basic)},

    /* calculate */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xea7090a0,
          "override",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Override,
          XFA_ScriptType::Basic)},

    /* print */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* extras */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Extras::type,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* proto */

    /* dSigData */

    /* creator */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* connect */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x24d85167,
          "timeout",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Timeout,
          XFA_ScriptType::Basic)},
    {ATTR(0x47d03490,
          "connection",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Connection,
          XFA_ScriptType::Basic)},
    {ATTR(0x552d9ad5,
          "usage",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usage,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc860f30a,
          "delayedOpen",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::DelayedOpen,
          XFA_ScriptType::Basic)},

    /* permissions */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* connectionSet */

    /* submit */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x28dee6e9,
          "format",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Format,
          XFA_ScriptType::Basic)},
    {ATTR(0x824f21b7,
          "embedPDF",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::EmbedPDF,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xdc75676c,
          "textEncoding",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TextEncoding,
          XFA_ScriptType::Basic)},
    {ATTR(0xf889e747,
          "xdpContent",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::XdpContent,
          XFA_ScriptType::Basic)},

    /* range */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* linearized */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* packet */
    {ATTR(0x97be91b,
          "content",
          &CJX_Packet::content,
          XFA_Attribute::Content,
          XFA_ScriptType::Basic)},

    /* rootElement */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* plaintextMetadata */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* numberSymbols */

    /* printHighQuality */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* driver */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* incrementalLoad */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* subjectDN */
    {ATTR(0x4156ee3f,
          "delimiter",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Delimiter,
          XFA_ScriptType::Basic)},

    /* compressLogicalStructure */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* incrementalMerge */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* radial */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* variables */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* timePatterns */

    /* effectiveInputPolicy */
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* nameAttr */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* conformance */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* transform */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* lockDocument */
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x2f16a382,
          "type",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Type,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* breakAfter */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x453eaf38,
          "startNew",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::StartNew,
          XFA_ScriptType::Basic)},
    {ATTR(0x9dcc3ab3,
          "trailer",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Trailer,
          XFA_ScriptType::Basic)},
    {ATTR(0xa6118c89,
          "targetType",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::TargetType,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc8da4da7,
          "target",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Target,
          XFA_ScriptType::Basic)},
    {ATTR(0xcbcaf66d,
          "leader",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Leader,
          XFA_ScriptType::Basic)},

    /* line */
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xabef37e3,
          "slope",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Slope,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},
    {ATTR(0xd996fa9b,
          "hand",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Hand,
          XFA_ScriptType::Basic)},

    /* list */
    {ATTR(0xa60dd202,
          "length",
          &CJX_List::length,
          XFA_Attribute::Length,
          XFA_ScriptType::Basic)},

    /* source */
    {ATTR(0x20146,
          "db",
          &CJX_Source::db,
          XFA_Attribute::Db,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* occur */
    {ATTR(0xb3543a6,
          "max",
          &CJX_Occur::max,
          XFA_Attribute::Max,
          XFA_ScriptType::Basic)},
    {ATTR(0xb356ca4,
          "min",
          &CJX_Occur::min,
          XFA_Attribute::Min,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x7d0b5fca,
          "initial",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Initial,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* pickTrayByPDFSize */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* monthNames */
    {ATTR(0x29418bb7,
          "abbr",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Abbr,
          XFA_ScriptType::Basic)},

    /* severity */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* groupParent */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* documentAssembly */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* numberSymbol */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},

    /* tagged */
    {ATTR(0xbe52dfbf,
          "desc",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Desc,
          XFA_ScriptType::Basic)},
    {ATTR(0xf6b47749,
          "lock",
          &CJX_Object::ScriptAttributeBool,
          XFA_Attribute::Lock,
          XFA_ScriptType::Basic)},

    /* items */
    {ATTR(0xbb8df5d,
          "ref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Ref,
          XFA_ScriptType::Basic)},
    {ATTR(0xc0811ed,
          "use",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Use,
          XFA_ScriptType::Basic)},
    {ATTR(0x570ce835,
          "presence",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Presence,
          XFA_ScriptType::Basic)},
    {ATTR(0xa5b410cf,
          "save",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Save,
          XFA_ScriptType::Basic)},
    {ATTR(0xbc254332,
          "usehref",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Usehref,
          XFA_ScriptType::Basic)},

    /* object */
    {ATTR(0xb2c80857,
          "className",
          &CJX_Object::className,
          XFA_Attribute::ClassName,
          XFA_ScriptType::Basic)},

    /* list */
    {ATTR(0xa60dd202,
          "length",
          &CJX_List::length,
          XFA_Attribute::Length,
          XFA_ScriptType::Basic)},

    /* [unknown] */

    /* tree */
    {ATTR(0x31b19c1,
          "name",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Name,
          XFA_ScriptType::Basic)},
    {ATTR(0x9f9d0f9,
          "all",
          &CJX_Tree::all,
          XFA_Attribute::All,
          XFA_ScriptType::Object)},
    {ATTR(0x4df15659,
          "nodes",
          &CJX_Tree::nodes,
          XFA_Attribute::Nodes,
          XFA_ScriptType::Object)},
    {ATTR(0x78a8d6cf,
          "classAll",
          &CJX_Tree::classAll,
          XFA_Attribute::ClassAll,
          XFA_ScriptType::Object)},
    {ATTR(0xcad6d8ca,
          "parent",
          &CJX_Tree::parent,
          XFA_Attribute::Parent,
          XFA_ScriptType::Object)},
    {ATTR(0xd5679c78,
          "index",
          &CJX_Tree::index,
          XFA_Attribute::Index,
          XFA_ScriptType::Basic)},
    {ATTR(0xdb5b4bce,
          "classIndex",
          &CJX_Tree::classIndex,
          XFA_Attribute::ClassIndex,
          XFA_ScriptType::Basic)},
    {ATTR(0xe4989adf,
          "somExpression",
          &CJX_Tree::somExpression,
          XFA_Attribute::SomExpression,
          XFA_ScriptType::Basic)},

    /* node */
    {ATTR(0x21aed,
          "id",
          &CJX_Object::ScriptAttributeString,
          XFA_Attribute::Id,
          XFA_ScriptType::Basic)},
    {ATTR(0x234a1,
          "ns",
          &CJX_Node::ns,
          XFA_Attribute::Ns,
          XFA_ScriptType::Basic)},
    {ATTR(0x50d1a9d1,
          "model",
          &CJX_Node::model,
          XFA_Attribute::Model,
          XFA_ScriptType::Object)},
    {ATTR(0xacb4823f,
          "isContainer",
          &CJX_Node::isContainer,
          XFA_Attribute::IsContainer,
          XFA_ScriptType::Basic)},
    {ATTR(0xe372ae97,
          "isNull",
          &CJX_Node::isNull,
          XFA_Attribute::IsNull,
          XFA_ScriptType::Basic)},
    {ATTR(0xfe612a5b,
          "oneOfChild",
          &CJX_Node::oneOfChild,
          XFA_Attribute::OneOfChild,
          XFA_ScriptType::Object)},

    /* [unknown] */

    /* [unknown] */

    /* model */
    {ATTR(0x97c1c65,
          "context",
          &CJX_Model::context,
          XFA_Attribute::Context,
          XFA_ScriptType::Object)},
    {ATTR(0x58be2870,
          "aliasNode",
          &CJX_Model::aliasNode,
          XFA_Attribute::AliasNode,
          XFA_ScriptType::Object)},

    /* [unknown] */
    {ATTR(0xa52682bd,
          "{default}",
          &CJX_Object::ScriptSomDefaultValue,
          XFA_Attribute::DefaultValue,
          XFA_ScriptType::Basic)},
    {ATTR(0xd6e27f1d,
          "value",
          &CJX_Object::ScriptSomDefaultValue,
          XFA_Attribute::Value,
          XFA_ScriptType::Basic)},
};

#undef ATTR

}  // namespace

ByteStringView XFA_AttributeValueToName(XFA_AttributeValue item) {
  return g_XFAEnumData[static_cast<int32_t>(item)].pName;
}

Optional<XFA_AttributeValue> XFA_GetAttributeValueByName(
    const WideStringView& name) {
  if (name.IsEmpty())
    return {};

  auto* it =
      std::lower_bound(std::begin(g_XFAEnumData), std::end(g_XFAEnumData),
                       FX_HashCode_GetW(name, false),
                       [](const XFA_AttributeValueInfo& arg, uint32_t hash) {
                         return arg.uHash < hash;
                       });
  if (it != std::end(g_XFAEnumData) && name.EqualsASCII(it->pName))
    return it->eName;

  return {};
}

Optional<XFA_SCRIPTATTRIBUTEINFO> XFA_GetScriptAttributeByName(
    XFA_Element eElement,
    WideStringView wsAttributeName) {
  if (wsAttributeName.IsEmpty())
    return {};

  uint32_t uHash = FX_HashCode_GetW(wsAttributeName, false);
  while (eElement != XFA_Element::Unknown) {
    const ScriptIndexRecord* scriptIndex =
        &g_ScriptIndexTable[static_cast<size_t>(eElement)];
    size_t iStart = scriptIndex->wAttributeStart;
    size_t iEnd = iStart + scriptIndex->wAttributeCount;
    for (size_t iter = iStart; iter < iEnd; ++iter) {
      const ElementAttributeRecord* pInfo = &g_ElementAttributeTable[iter];
      if (uHash == pInfo->uHash) {
        XFA_SCRIPTATTRIBUTEINFO result;
        result.attribute = pInfo->attribute;
        result.eValueType = pInfo->eValueType;
        result.callback = pInfo->callback;
        return result;
      }
    }
    eElement = g_XFAScriptParents[static_cast<size_t>(eElement)];
  }
  return {};
}
